@@ -21,6 +21,8 @@
 // Models
 #include "models/historymodel.h"
 
+#include "controllers/server.h"
+
 #define NOTA_URI "org.maui.nota"
 
 Q_DECL_EXPORT int main(int argc, char *argv[])
@@ -39,8 +41,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     app.setOrganizationName(QStringLiteral("Maui"));
     app.setWindowIcon(QIcon(":/img/nota.svg"));
 
-    MauiApp::instance()->setIconName("qrc:/img/nota.svg");
-
     KLocalizedString::setApplicationDomain("nota");
 
     KAboutData about(QStringLiteral("nota"), i18n("Nota"), NOTA_VERSION_STRING, i18n("Browse, create and edit text files."), KAboutLicense::LGPL_V3,i18n("© 2019-%1 Maui Development Team", QString::number(QDate::currentDate().year())), QString(GIT_BRANCH) + "/" + QString(GIT_COMMIT_HASH));
@@ -54,6 +54,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     about.setProgramLogo(app.windowIcon());
 
     KAboutData::setApplicationData(about);
+    MauiApp::instance()->setIconName("qrc:/img/nota.svg");
 
     QCommandLineParser parser;
 
@@ -62,6 +63,33 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 
     about.processCommandLine(&parser);
     const QStringList args = parser.positionalArguments();
+    QStringList paths;
+
+    if (!args.isEmpty())
+    {
+        for(const auto &path : args)
+            paths << QUrl::fromUserInput(path).toString();
+    }
+
+#if (defined Q_OS_LINUX || defined Q_OS_FREEBSD) && !defined Q_OS_ANDROID
+    if (AppInstance::attachToExistingInstance(QUrl::fromStringList(paths), false))
+    {
+        // Successfully attached to existing instance of Nota
+        return 0;
+    }else
+    {
+        //        const QString serviceName = QStringLiteral("org.kde.index-%1").arg(QCoreApplication::applicationPid());
+        //        auto instances = IndexInstance::appInstances(serviceName);
+        //        if(instances.size() > 0)
+        //        {
+        //            instances.first().first->activateWindow();
+        //        }
+    }
+
+    AppInstance::registerService();
+#endif
+
+    auto server = std::make_unique<Server>();
 
     QQmlApplicationEngine engine;
     const QUrl url(QStringLiteral("qrc:/main.qml"));
@@ -69,17 +97,22 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
                 &engine,
                 &QQmlApplicationEngine::objectCreated,
                 &app,
-                [url, args](QObject *obj, const QUrl &objUrl) {
+                [url, args, &server](QObject *obj, const QUrl &objUrl) {
         if (!obj && url == objUrl)
             QCoreApplication::exit(-1);
 
-        if (!args.isEmpty())
-            Nota::instance()->requestFiles(args);
+        server->setQmlObject(obj);
+        server->openFiles(args, false);
+
     },
     Qt::QueuedConnection);
 
+    engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
+
+    qmlRegisterSingletonInstance<Server>(NOTA_URI, 1, 0, "Server", server.get());
     qmlRegisterSingletonInstance<Nota>(NOTA_URI, 1, 0, "Nota", Nota::instance());
     qmlRegisterType<HistoryModel>(NOTA_URI, 1, 0, "History");
+
     engine.load(url);
 
     return app.exec();
